//
// Java keystore generation tool
// 2004-11-24 Martin Bartosch, Cynops GmbH <m.bartosch@cynops.de>
//

/*
 Tool for Java Keystore generation. In contrast to keytool this program
 is able to import an RSA private key to the keystore as well.

  Usage: java de.cynops.java.crypto.keystore.CreateKeystore \
	 -keystore keystore.server \
	 -storepass <keystore password> \
	 -cert LABEL:<cert file> [-cert ...] \
	 -keypass <keystore password for RSA private key> \
	 -key LABEL:<RSA private key file in PKCS#8 format>

 The PKCS#8 key must NOT be encrypted.

 As an alternative it is possible to call the program with a single
 dash as argument. In this case, the program reads its parameters from
 command line and then from STDIN. Parameters specified on STDIN override
 those specified on the command line (does not apply to cert= arguments,
 these are appended to the list of certificates)

 STDIN is read line by line, each line contains one single setting.
 Arguments are set with the general syntax
 key=value

 Example:
 java de.cynops.java.crypto.keystore.CreateKeystore -
keystore=keystore.server
storepass=VerySecret
cert=CERT:cert.der
cert=CACERT:cacert.der
cert=ROOTCACERT:rootcacert.der
keypass=EvenMoreSecret
key=RSAKEY:key.p8

 If the key filename is empty then the private key is expected as binary
 input on STDIN immediately after the newline following the colon.
 Example:
keypass=EvenMoreSecret
key=RSAKEY:
<binary key data...>

Known bugs:
This Program Is Ugly.

*/

package de.cynops.java.crypto.keystore;

import java.io.*;
import java.util.*;
import java.security.*;
import java.security.cert.*;
import java.security.spec.*;
import java.lang.System.*;

public class CreateKeystore {

    public static java.security.cert.Certificate getcert(String file)
	throws Exception {
	InputStream is = new FileInputStream(file);
	CertificateFactory certfact = CertificateFactory.getInstance("X.509");
	X509Certificate cert = 
	    (X509Certificate) certfact.generateCertificate(is);

	return cert;
    }


    // byte array to private key
    public static PrivateKey getprivatekey(byte[] buffer)
	throws java.security.GeneralSecurityException {
	try {
	    KeyFactory kfact = KeyFactory.getInstance("RSA");
	    PKCS8EncodedKeySpec keyspec = 
		new PKCS8EncodedKeySpec(buffer);	    
	    PrivateKey privkey = kfact.generatePrivate (keyspec);
	    return privkey;
	} catch (java.security.GeneralSecurityException e) {
	    throw e;
	}
    }
    
    // read private key from Stream
    public static PrivateKey getprivatekey(int size, InputStream stream) 
	throws Exception {

	try {
	    byte[] buffer = new byte[size];
	    int nrofbytes = stream.read(buffer, 0, size);
	    return (getprivatekey(buffer));
	} catch (IOException e) {
	    throw e;
	}
    }

    // read private key from Reader (1:1 conversion from char to byte required)
    public static PrivateKey getprivatekey(int size, Reader reader) 
	throws Exception {

	try {
	    char[] charbuffer = new char[size];
	    byte[] buffer = new byte[size];
	    int nrofbytes = reader.read(charbuffer, 0, size);
	    
	    for (int ii = 0; ii < nrofbytes; ii++) {
		buffer[ii] = (byte) charbuffer[ii];
	    }

	    return (getprivatekey(buffer));
	} catch (IOException e) {
	    throw e;
	}
    }
    


    // split label out of argument
    // arg: "foo" -> return defaultlabel
    // arg: "foo:bar" -> return "foo"
    // arg: "foo:" -> return "foo"
    public static String getlabel(String defaultlabel, String arg) {
	StringTokenizer s = 
	    new StringTokenizer(arg, ":");
	String label;

	if (s.hasMoreTokens())
	    label = s.nextToken();
	else
	    return null;
	
	if (s.hasMoreTokens() || (arg.endsWith(":")))
	    return(label);
	else
	    return(defaultlabel);
    }

    // split filename out of argument
    // arg: "foo" -> return "foo"
    // arg: "foo:bar" -> return "bar"
    // arg: "foo:" -> return ""
    public static String getfilename(String arg) {
	StringTokenizer s = 
	    new StringTokenizer(arg, ":");
	String label;

	if (s.hasMoreTokens())
	    label = s.nextToken();
	else
	    return null;
	
	if (s.hasMoreTokens())
	    return(s.nextToken());
	else
	    if (arg.endsWith(":"))
		return("");
	    return(label);
    }

    // add specified cert to list
    public static boolean addcert(List certs, String arg) 
	throws Exception {
	try {
	    CertInfo certinfo = new CertInfo();
	    certinfo.label = getlabel("CERT" + certs.size(), arg);
	    certinfo.certificate = getcert(getfilename(arg));
	    certs.add(certinfo);
	} catch (Exception e) {
	    throw e;
	}
	    
	return true;
    }


    public static void main(String[] args) {
	try {
	    
	    String ksFilename = null;
	    String storePassword = null;
	    String keyFilename = null;
	    String keyLabel = null;
	    String keyPassword = null;
	    boolean fromstdin = false;
	    List certs = new ArrayList();

	    for(int ii = 0; ii < args.length; ii++) {
		if(args[ii].equalsIgnoreCase("-")) {
		    ii++;
		    fromstdin = true;
		    continue;
		}
		
		if(args[ii].equalsIgnoreCase("-keystore")) {
		    ii++;
		    ksFilename = new String(args[ii]);
		    continue;
		}
		
		if(args[ii].equalsIgnoreCase("-storepass")) {
		    ii++;
		    storePassword = new String(args[ii]);
		    continue;
		}

		if(args[ii].equalsIgnoreCase("-key")) {
		    ii++;
		    keyLabel = getlabel("KEY", args[ii]);
		    keyFilename = getfilename(args[ii]);
		    continue;
		}
		
		if(args[ii].equalsIgnoreCase("-keypass")) {
		    ii++;
		    keyPassword = args[ii];
		    continue;
		}

		if(args[ii].equalsIgnoreCase("-cert")) {
		    ii++;
		    if (!addcert(certs, args[ii])) {
			System.err.println("Invocation error");
			System.exit(1);
		    }
		    continue;
		}
	    }

	    // problem:
	    // - we need a Reader for processing STDIN line by line
	    // - we also need to process binary data from STDIN (P8 key)
	    // - binary data is usually not handled by readers due to
	    //   character conversions
	    // remedy
	    // - force 8 bit character set (ISO-8859-1) for the reader
	    // - when reading binary data we have to convert char[] to
	    //   buffer[] before processing key data
	    //
	    // This Is Ugly, but I do not see a suitable way around it.
	    //
	    BufferedReader stdin = 
		new BufferedReader(new InputStreamReader(System.in,
							 "ISO-8859-1"));

	    if (fromstdin) {
		String line;
		
		while ((line = stdin.readLine()) != null) {
		    int sep = line.indexOf('=');
		    if (sep != -1) {
			String key = line.substring(0, sep);
			String value = line.substring(sep + 1, line.length());
			if (key.equalsIgnoreCase("keystore"))
			    ksFilename = value;
			if (key.equalsIgnoreCase("storepass"))
			    storePassword = value;
			if (key.equalsIgnoreCase("keypass"))
			    keyPassword = value;
			if (key.equalsIgnoreCase("cert")) {
			    if (! addcert(certs, value)) {
				System.err.println("Invocation error");
				System.exit(1);
			    }
			}
			if (key.equalsIgnoreCase("key")) {
			    keyLabel = getlabel("KEY", value);
			    keyFilename = getfilename(value);
			    if (keyFilename.equals(""))
				break;
			}
		    }
		}
	    }

	    if(ksFilename == null) {
		ksFilename = new String(System.getProperty("user.home")
					+ System.getProperty("file.separator")
					+ ".keystore");
	    }

	    if(keyLabel == null) {
		System.err.println("No RSA key specified.");
		System.exit(1);
	    }

	    
	    System.err.println("Creating keystore " + ksFilename + ".");
	    
	    if (storePassword == null)
		storePassword = "";
	    if (keyPassword == null)
		keyPassword = "";
	    

	    KeyStore store = KeyStore.getInstance("JKS");
	    store.load(null, storePassword.toCharArray());

	    java.security.cert.Certificate[] certchain = 
		new java.security.cert.Certificate[certs.size()];
	    
	    int ii = 0;
	    // add certificates
	    for (Iterator i = certs.iterator();
		 i.hasNext(); ) {
		CertInfo certinfo = (CertInfo) i.next();
		store.setCertificateEntry(certinfo.label,
					  certinfo.certificate);
		certchain[ii] = certinfo.certificate;
		ii++;
	    }

	    if ((keyFilename != null) && (! keyFilename.equals(""))) {
		// add private key from file and cert chain
		FileInputStream inputstream = null;
		DataInputStream datainputstream = null;
		inputstream = new FileInputStream(keyFilename);
		datainputstream = new DataInputStream(inputstream);

		PrivateKey privatekey = 
		    getprivatekey(inputstream.available(),
				  datainputstream);
		
		datainputstream.close();
		inputstream.close();

		
		store.setKeyEntry(keyLabel,
				  privatekey, 
				  keyPassword.toCharArray(), 
				  certchain);
	    } else {
		// add private key from stdin and cert chain

		PrivateKey privatekey = 
		    getprivatekey(100000,  // uh...
				  stdin);
		
		
		store.setKeyEntry(keyLabel,
				  privatekey, 
				  keyPassword.toCharArray(), 
				  certchain);

	    }
	    

	    stdin.close();

	    FileOutputStream ksout = new FileOutputStream(ksFilename);
            store.store(ksout, storePassword.toCharArray());
            ksout.flush();
	
	} catch (Exception ex) {
	    ex.printStackTrace();
	    System.exit(1);
	}
	
    }
    
}
